/**
* This file is part of the QBittorrent Project
*
* Bittorrent Client using Qt4 and libtorrent.
* Copyright (C) 2006  Christophe Dumez
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public License
* along with this library; see the file COPYING.LIB.  If not, write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301, USA.
**/

#include "screenedsearch.h"
#include "iconprovider.h"

/* QtCore */
#include <QtCore/QDebug>
#include <QtCore/QStringList>

/* QtGui */
#include <QtGui/QHBoxLayout>
#include <QtGui/QIcon>
#include <QtGui/QLabel>
#include <QtGui/QToolButton>
#include <QtGui/QVBoxLayout>

ScreenedSearch::ScreenedSearch ( QWidget * parent )
    : QWidget ( parent )
    , minRange ( 2 )
{
  setObjectName ( QLatin1String ( "ScreenedSearch" ) );

  // main layout
  QVBoxLayout* verticalLayout = new QVBoxLayout ( this );

  // groupbox
  m_groupBox = new QGroupBox ( this );
  m_groupBox->setTitle ( trUtf8 ( "filter current search result" ) );
  verticalLayout->addWidget ( m_groupBox );

  // groupbox layout
  QHBoxLayout* boxLayout = new QHBoxLayout ( m_groupBox );

  // clear Button
  QToolButton* m_toolButton = new QToolButton ( m_groupBox );
  m_toolButton->setToolTip ( trUtf8 ( "clear" ) );
  boxLayout->addWidget ( m_toolButton );

  QIcon clearIcon = IconProvider::instance()->getIcon ( "edit-clear-locationbar-ltr" );
  if ( clearIcon.isNull() )
    m_toolButton->setArrowType ( Qt::RightArrow );
  else
    m_toolButton->setIcon ( clearIcon );


  // search input
  m_lineEdit = new QLineEdit ( m_groupBox );
  m_lineEdit->setToolTip ( trUtf8 ( "input" ) );
  m_lineEdit->setWhatsThis ( trUtf8 ( "This filter always provides regular expression for exclude unmatched inputs from search result" ) );
  boxLayout->addWidget ( m_lineEdit );

  // search Button
  QToolButton* m_executeButton = new QToolButton ( m_groupBox );
  m_executeButton->setIcon ( IconProvider::instance()->getIcon ( "edit-find" ) );
  m_executeButton->setToolTip ( trUtf8 ( "start filter" ) );
  boxLayout->addWidget ( m_executeButton );

  // set minimum minRange length before send full string
  m_spinBox = new QSpinBox ( m_groupBox );
  m_spinBox->setRange ( minRange, 99 );
  boxLayout->addWidget ( m_spinBox );

  // info label
  QLabel* infoLabel = new QLabel ( m_groupBox );
  infoLabel->setText ( trUtf8 ( "start filtering with minimum text length" ) );
  boxLayout->addWidget ( infoLabel );

  boxLayout->addStretch ( 0 );

  // finalize layout
  setLayout ( verticalLayout );

  // signals
  connect ( m_toolButton, SIGNAL ( clicked() ),
            m_lineEdit, SLOT ( clear() ) );

  connect ( m_lineEdit, SIGNAL ( textChanged ( const QString & ) ),
            this, SLOT ( searchTextChanged ( const QString & ) ) );

  connect ( m_executeButton, SIGNAL ( clicked() ), this, SLOT ( execute() ) );

  connect ( m_spinBox, SIGNAL ( valueChanged ( int ) ),
            this, SLOT ( setMinimumLength ( int ) ) );
}

QString ScreenedSearch::preparePattern ( const QString &input ) const
{
  QString buffer ( input );
  buffer.trimmed();

  return buffer;
}

void ScreenedSearch::searchTextChanged ( const QString &txt )
{
  QString buffer = preparePattern ( txt );
  if ( buffer.length() < minRange )
  {
    emit prepareSearchResult ( QRegExp() );
    return;
  }

  qDebug() << Q_FUNC_INFO << buffer;
  QRegExp regexp ( buffer, Qt::CaseInsensitive );
  emit prepareSearchResult ( regexp );
  buffer.clear();
}

void ScreenedSearch::setMinimumLength ( int c )
{
  minRange = c;
  searchTextChanged ( m_lineEdit->text() );
}

void ScreenedSearch::execute()
{
  searchTextChanged ( m_lineEdit->text() );
}

int ScreenedSearch::getMinimumRange()
{
  return m_spinBox->value();
}

void ScreenedSearch::setMinimumRange ( int c )
{
  if ( c >= m_spinBox->value() )
  {
    minRange = c;
    m_spinBox->setMinimum ( c );
  }
}

ScreenedSearch::~ScreenedSearch()
{
  m_lineEdit->clear();
}
