/**
* This file is part of the QBittorrent Project
*
* Bittorrent Client using Qt4 and libtorrent.
* Copyright (C) 2006  Christophe Dumez
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public License
* along with this library; see the file COPYING.LIB.  If not, write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301, USA.
**/

#ifndef SCREENEDSEARCH_H
#define SCREENEDSEARCH_H

/* QtCore */
#include <QtCore/QMetaObject>
#include <QtCore/QObject>
#include <QtCore/QRegExp>
#include <QtCore/QString>

/* QtGui */
#include <QtGui/QGroupBox>
#include <QtGui/QLineEdit>
#include <QtGui/QSpinBox>
#include <QtGui/QWidget>

/**
*  filtering the given search result from \class SearchEngine
*
* \class ScreenedSearch
* \short filtering search results
* \author Juergen Heinemann (Undfined) http://www.hjcms.de
* \since 2011/05/06
* \version 2.8.0
*
* \code
// Binding example:
SearchEngine::SearchEngine(MainWindow *parent) : QWidget(parent), mp_mainWindow(parent) {
{
  ...

  ScreenedSearch* m_ScreenedSearch = new ScreenedSearch ( this );
  // myLayout->addWidget(m_ScreenedSearch);

  ...

  connect ( m_ScreenedSearch, SIGNAL ( prepareSearchResult ( const QRegExp & ) ),
                 this, SLOT ( visibleSearchResult ( const QRegExp & ) ) );
}

void SearchEngine::visibleSearchResult ( const QRegExp &reg )
{
  if (tabWidget->currentIndex() < 0)
      return; // FIXME

  QSortFilterProxyModel* model = all_tab.at(tabWidget->currentIndex())->getCurrentSearchListProxy();
  model->setFilterRegExp( reg );
  model->sort( NAME, Qt::AscendingOrder );
}
* \endcode
*/
class ScreenedSearch : public QWidget
{
    Q_OBJECT
    Q_CLASSINFO ( "Author", "Jürgen Heinemann (Undefined)" )
    Q_CLASSINFO ( "URL", "http://www.hjcms.de" )
    /** this property set the minimum minRange length startpoint and also set the minimum range for \ref m_spinBox */
    Q_PROPERTY ( int minRange READ getMinimumRange WRITE setMinimumRange )

  private:
    /** start with minimum length */
    int minRange;

    QGroupBox* m_groupBox;

    /** Filter Input */
    QLineEdit* m_lineEdit;

    /** Setting the minimum \ref minRange length */
    QSpinBox* m_spinBox;

    inline QString preparePattern ( const QString & ) const;

  private Q_SLOTS:
    /** this slot is triggered when \ref m_lineEdit signal:textChanged emitted */
    void searchTextChanged ( const QString &txt );

    /** this slot is triggered when \ref m_spinBox signal:valueChanged emitted */
    void setMinimumLength ( int );

  Q_SIGNALS:
    /**
    * this signal is always emitted whenever the text from linedit has changed!
    * if input length is >=  \ref minRange, it will send the given data otherwise
    * it send's an empty string.
    *
    * \param reg  filter regexp for current search result
    */
    void prepareSearchResult ( const QRegExp &reg );

  public Q_SLOTS:
    /**
    * send current value from linedit depends on \ref searchTextChanged
    */
    Q_SCRIPTABLE void execute ();

  public:
    /** Search Filter constructor */
    ScreenedSearch ( QWidget * parent = 0 );

    /**
    * get current restricted text length
    */
    Q_SCRIPTABLE int getMinimumRange();

    /**
    *  set minimum required text length before trigger \ref prepareSearchResult
    *  \note it will also modify the current SpinBox value \see setMinimumLength
    *  \param c  the default value is minimum 2 minRange
    */
    Q_SCRIPTABLE void setMinimumRange ( int c );

    /** Search Filter destructor */
    virtual ~ScreenedSearch();
};

#endif
